@@ -42,12 +42,6 @@ extern "C" {
 #define HAS_SV2OBJ
 #endif
 
-#if PERL_VERSION < 12
-#   define SVt_RV_FAKE SVt_RV
-#else
-#   define SVt_RV_FAKE SVt_IV
-#endif
-
 #ifndef NDEBUG
 #   if DEBUG > 1
 #       define TRACE_READER     1
@@ -68,6 +62,13 @@ extern "C" {
 #   define SRL_PATH_TRACE(msg, args...)
 #endif
 
+#define CLEAN_RESULTS(p) STMT_START {   \
+    if ((p)->results) {                 \
+        SvREFCNT_dec((p)->results);     \
+        (p)->results = NULL;            \
+    }                                   \
+} STMT_END
+
 SRL_STATIC_INLINE void srl_parse_next(pTHX_ srl_path_t *path, int expr_idx, SV *route);
 SRL_STATIC_INLINE void srl_parse_next_int(pTHX_ srl_path_t *path, int expr_idx, SV *route, IV n);
 SRL_STATIC_INLINE void srl_parse_next_str(pTHX_ srl_path_t *path, int expr_idx, SV *route,
@@ -110,8 +111,7 @@ srl_build_path_struct(pTHX_ HV *opt)
 void
 srl_destroy_path(pTHX_ srl_path_t *path)
 {
-    if (path->results)
-        SvREFCNT_dec(path->results);
+    CLEAN_RESULTS(path);
 
     if (path->iter)
         srl_destroy_iterator(aTHX_ path->iter);
@@ -124,15 +124,11 @@ void
 srl_path_reset(pTHX_ srl_path_t *path, SV *src)
 {
     path->expr = NULL;
+    CLEAN_RESULTS(path);
 
     if (path->iter) srl_destroy_iterator(aTHX_ path->iter);
     path->iter = srl_build_iterator_struct(aTHX_ NULL);
 
-    if (path->results) {
-        SvREFCNT_dec(path->results);
-        path->results = NULL;
-    }
-
     if (sv_isa(src, "Sereal::Path::Iterator")) {
         croak("not implemented");
     } else {
@@ -144,19 +140,15 @@ void
 srl_path_traverse(pTHX_ srl_path_t *path, AV *expr, SV *route)
 {
     SV *route_copy;
-    if (!path->iter)
-        croak("Set Iter first"); // TODO
+    if (!path->iter) croak("No document to traverse");
 
     assert(expr != NULL);
     assert(route != NULL);
 
-    if (path->results) {
-        SvREFCNT_dec(path->results);
-        path->results = NULL;
-    }
+    CLEAN_RESULTS(path);
 
     path->results = newAV();
-    path->expr = expr; // TODO perhaps, copy expr
+    path->expr = expr;
     route_copy = sv_2mortal(newSVsv(route));
 
     srl_iterator_reset(aTHX_ path->iter);
@@ -166,21 +158,9 @@ srl_path_traverse(pTHX_ srl_path_t *path, AV *expr, SV *route)
 SV *
 srl_path_results(pTHX_ srl_path_t *path)
 {
-    SV *into, *referent;
-    AV *results = path->results;
-    if (!results)
-        return sv_2mortal(newSV_type(SVt_NULL));
-
+    AV *results = path->results ? path->results : newAV();
     path->results = NULL;
-    referent = (SV*) results;
-    into = sv_2mortal(newSV_type(SVt_NULL));
-
-    // copy-pasted from decoder
-    sv_upgrade(into, SVt_RV_FAKE);
-    SvTEMP_off(referent);
-    SvRV_set(into, referent);
-    SvROK_on(into);
-    return into;
+    return sv_2mortal(newRV_noinc((SV*) results));
 }
 
 SRL_STATIC_INLINE void
@@ -196,7 +176,7 @@ srl_parse_next(pTHX_ srl_path_t *path, int expr_idx, SV *route)
         SV *res;
         print_route(route, "to decode");
         res = srl_iterator_decode(aTHX_ iter);
-        SvREFCNT_inc(res); // TODO ????
+        SvREFCNT_inc(res);
         av_push(path->results, res); // TODO store route if needed
         return;
     }
@@ -429,10 +409,7 @@ srl_parse_array_range(pTHX_ srl_path_t *path, int expr_idx, SV *route, int *rang
     stop  = stop  < 0 ? MAX(0, stop  + stack->count) : MIN(stack->count, stop);
     step  = step ? step : 1;
 
-    if (step < 0) {
-        warn("nagative step in not supported"); // TODO
-        return;
-    }
+    if (step < 0) croak("negative step in not supported");
 
     SRL_PATH_TRACE("parse items '%d:%d:%d' in array of size=%d at depth=%"IVdf,
                    start, stop, step, stack->count, expected_depth);
